package com.sqx.modules.passenger.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.sqx.common.utils.Result;
import com.sqx.modules.app.annotation.Login;
import com.sqx.modules.common.service.CommonInfoService;
import com.sqx.modules.passenger.entity.Passenger;
import com.sqx.modules.passenger.service.PassengerService;
import com.sqx.modules.userRealnameInfo.service.UserRealnameInfoService;
import com.sqx.modules.utils.AESCBCWithRandomIV;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.List;

@RestController
@Api(value = "乘客-用户端", tags = {"乘客-用户端"})
@RequestMapping("/app/passenger")
public class PassengerController {

    @Resource
    private PassengerService passengerService;
    @Resource
    private CommonInfoService commonInfoService;
    @Resource
    private UserRealnameInfoService userRealnameInfoService;

    @Login
    @GetMapping("/selectList")
    @ApiOperation("查询乘客")
    public Result selectList(@RequestAttribute("userId")Long userId){
        String key = commonInfoService.findOne(819).getValue();
        List<Passenger> list = passengerService.list(new QueryWrapper<Passenger>().eq("user_id", userId));
        if (list.size() > 0 ){
            for (Passenger l:list){
                //解密身份证号
                String decrypt = null;
                try {
                    decrypt = AESCBCWithRandomIV.decrypt(l.getIdCard(), key);
                } catch (Exception e) {
                    e.printStackTrace();
                }
                l.setIdCard(userRealnameInfoService.maskIdKeepFront(decrypt));
            }
        }
        return Result.success().put("data",list);
    }

    @Login
    @PostMapping("/insert")
    @ApiOperation("新增乘客")
    public Result insert(@RequestAttribute("userId")Long userId,Passenger passenger){
        String key = commonInfoService.findOne(819).getValue();
        String encrypt = null;
        try {
            encrypt = AESCBCWithRandomIV.encrypt(passenger.getIdCard(), key);
        } catch (Exception e) {
            e.printStackTrace();
        }
        passenger.setUserId(userId);
        passenger.setIdCard(encrypt);
        passengerService.insert(passenger);
        return Result.success();
    }

    @PostMapping("/update")
    @ApiOperation("修改乘客")
    public Result update(Passenger passenger){
        String key = commonInfoService.findOne(819).getValue();
        if (null != passenger.getIdCard() && passenger.getIdCard() != ""){
            String encrypt = null;
            try {
                encrypt = AESCBCWithRandomIV.encrypt(passenger.getIdCard(), key);
            } catch (Exception e) {
                e.printStackTrace();
            }
            passenger.setIdCard(encrypt);
        }
        return Result.success().put("data",passengerService.updateById(passenger));
    }

    @PostMapping("/delete")
    @ApiOperation("删除乘客")
    public Result delete(Long id){
        return Result.success().put("data",passengerService.removeById(id));
    }
}
