package com.sqx.modules.utils;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.google.common.collect.Maps;
import com.sqx.modules.common.service.CommonInfoService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.web.client.RestTemplate;

import javax.imageio.ImageIO;
import javax.servlet.http.HttpServletResponse;
import java.awt.image.BufferedImage;
import java.io.InputStream;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.net.URL;
import java.net.URLConnection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Slf4j
@Component
public class SenInfoCheckUtil {

    private static Logger logger = LoggerFactory.getLogger(SenInfoCheckUtil.class);

    private static String MpAccessToken;

    private static String GZHAccessToken;

    private static String RiderGZHAccessToken;

    // 这里使用静态，让 service 属于类
    private static CommonInfoService commonInfoService;


    // 注入的时候，给类的 service 注入
    @Autowired
    public void setWxChatContentService(CommonInfoService commonInfoService) {
        SenInfoCheckUtil.commonInfoService = commonInfoService;
    }


    /**
     * 获取Token  小程序
     * @param
     * @param
     * @return AccessToken
     */
    public static String getMpToken(){
        if(StringUtils.isEmpty(MpAccessToken)){
            getMpAccessToken();
        }
        return MpAccessToken;
    }

    /**
     * 获取Token  小程序
     * @param
     * @param
     * @return AccessToken
     */
    public static String getMpRiderToken(){
        /*if(StringUtils.isEmpty(MpAccessToken)){
            getMpAccessToken();
        }*/
        //   return getMpAccessToken();
        return getRiderMpAccessToken();
    }

    public static String getGZHToken() {
        return getGZHAccessToken();
    }

    public static String getRiderGZHToken() {
        return getRiderGZHAccessToken();
    }

    public static String getGZHAccessToken() {
        String appid = commonInfoService.findOne(5).getValue();
        String secret = commonInfoService.findOne(21).getValue();
        String jsonResult = HttpClientUtil.doPost("https://api.weixin.qq.com/cgi-bin/token?grant_type=client_credential&appid=" + appid + "&secret=" + secret);
        JSONObject parseObject = JSON.parseObject(jsonResult);
        logger.info("=========accessTokenOut=========" + parseObject.toJSONString());

        String errcode = parseObject.getString("errcode");
        String accessToken = parseObject.getString("access_token");
        String expiresIn = parseObject.getString("expires_in");
        return accessToken;
    }

    public static String getRiderGZHAccessToken() {
        String appid = commonInfoService.findOne(262).getValue();
        String secret = commonInfoService.findOne(263).getValue();
        String jsonResult = HttpClientUtil.doPost("https://api.weixin.qq.com/cgi-bin/token?grant_type=client_credential&appid=" + appid + "&secret=" + secret);
        JSONObject parseObject = JSON.parseObject(jsonResult);
        logger.info("=========accessTokenOut=========" + parseObject.toJSONString());

        String errcode = parseObject.getString("errcode");
        String accessToken = parseObject.getString("access_token");
        String expiresIn = parseObject.getString("expires_in");
        return accessToken;
    }


    public static void getImg(String relation,String goodsId,String type, String page,HttpServletResponse response){
        String mpToken = getMpToken();
        //获取二维码数据
        String url = "https://api.weixin.qq.com/wxa/getwxacodeunlimit?access_token="+mpToken;
        Map<String,Object> map = Maps.newHashMap();
        map.put("scene",relation+"&"+goodsId+"&"+type);
        String value = commonInfoService.findOne(105).getValue();
        if("是".equals(value)){
            map.put("page",page);
        }
        map.put("width", 280);
        String jsonString = JSON.toJSONString(map);
        InputStream inputStream = sendPostBackStream(url, jsonString);
        //生成二维码图片
        response.setContentType("image/png");
        try{
            BufferedImage bi = ImageIO.read(inputStream);
            ImageIO.write(bi, "JPG", response.getOutputStream());
            inputStream.close();
        }catch (Exception e){
            e.printStackTrace();
        }
    }


    /**
     * 获取二维码图片
     */
    public static void getPoster(String invitationCode, HttpServletResponse response){
        String mpToken = getMpToken();
        //获取二维码数据
        String url = "https://api.weixin.qq.com/wxa/getwxacodeunlimit?access_token="+mpToken;
        Map<String,Object> map = Maps.newHashMap();
        map.put("scene",invitationCode);
        map.put("width", 280);
        String jsonString = JSON.toJSONString(map);
        InputStream inputStream = sendPostBackStream(url, jsonString);
        //生成二维码图片
        response.setContentType("image/png");
        try{
            BufferedImage bi = ImageIO.read(inputStream);
            ImageIO.write(bi, "JPG", response.getOutputStream());
            inputStream.close();
        }catch (Exception e){
            logger.error(e.getMessage());
        }
    }

    public static void riderQr(Long userId, HttpServletResponse response) {
        String mpToken = getMpToken();
        //获取二维码数据
        String url = "https://api.weixin.qq.com/wxa/getwxacodeunlimit?access_token="+mpToken;
        Map<String,Object> map = Maps.newHashMap();
        map.put("scene","&userId="+userId);
        map.put("width", 280);
        String jsonString = JSON.toJSONString(map);
        InputStream inputStream = sendPostBackStream(url, jsonString);
        //生成二维码图片
        response.setContentType("image/png");
        try{
            BufferedImage bi = ImageIO.read(inputStream);
            ImageIO.write(bi, "JPG", response.getOutputStream());
            inputStream.close();
        }catch (Exception e){
            logger.error(e.getMessage());
        }
    }
    private static InputStream sendPostBackStream(String url, String param) {
        PrintWriter out = null;
        try {
            URL realUrl = new URL(url);
            // 打开和URL之间的连接
            URLConnection conn = realUrl.openConnection();
            // 设置通用的请求属性
            conn.setRequestProperty("accept", "*/*");
            conn.setRequestProperty("connection", "Keep-Alive");
            conn.setRequestProperty("user-agent",
                    "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1;SV1)");
            conn.setDoOutput(true);
            conn.setDoInput(true);
            //解决乱码问题
            OutputStreamWriter outWriter =new OutputStreamWriter(conn.getOutputStream(), "utf-8");
            out =new PrintWriter(outWriter);
            // 发送请求参数
            if(StringUtils.isNotBlank(param)) {
                out.print(param);
            }
            // flush输出流的缓冲
            out.flush();
            return conn.getInputStream();
        } catch (Exception e) {
            logger.error("发送 POST 请求出现异常！"+e);
        } finally{
            IOUtils.closeQuietly(out);
        }
        return null;
    }


    /**
     * 获取access_token
     * 每个两个小时自动刷新AcessTocken
     */
    @Scheduled(cron = "0 0 */2 * * ?")
    public static void getMpAccessToken(){
        String appid = commonInfoService.findOne(45).getValue();
        String secret = commonInfoService.findOne(46).getValue();
        String jsonResult = HttpClientUtil.doPost("https://api.weixin.qq.com/cgi-bin/token?grant_type=client_credential&appid=" + appid + "&secret=" + secret);
        JSONObject parseObject = JSON.parseObject(jsonResult);
        logger.info("=========accessTokenOut========="+parseObject.toJSONString());

        String errcode = parseObject.getString("errcode");
        String accessToken = parseObject.getString("access_token");
        String expiresIn = parseObject.getString("expires_in");
        MpAccessToken=accessToken;
    }

    /**
     * 获取access_token
     * 每个两个小时自动刷新AcessTocken
     */
    @Scheduled(cron = "0 0 */2 * * ?")
    public static String getRiderMpAccessToken(){
        String appid = commonInfoService.findOne(248).getValue();
        String secret = commonInfoService.findOne(249).getValue();
        String jsonResult = HttpClientUtil.doPost("https://api.weixin.qq.com/cgi-bin/token?grant_type=client_credential&appid=" + appid + "&secret=" + secret);
        JSONObject parseObject = JSON.parseObject(jsonResult);
        logger.info("=========accessTokenOut========="+parseObject.toJSONString());

        String errcode = parseObject.getString("errcode");
        String accessToken = parseObject.getString("access_token");
        String expiresIn = parseObject.getString("expires_in");
        return accessToken;
    }

    /**
     * 小程序消息推送
     * @param wxId  用户openId
     * @param templateId  模板id
     * @param page   页面路由
     * @param msgList  消息内容集合
     * @param type   类型
     */
    public static void sendMsg(String wxId, String templateId, List<String> msgList, Integer type){
        String mpToken = getMpToken();
        RestTemplate restTemplate = new RestTemplate();
        String url = "https://api.weixin.qq.com/cgi-bin/message/subscribe/send?access_token=" +mpToken;
        //拼接推送的模版
        WxMssVo wxMssVo = new WxMssVo();
        wxMssVo.setTouser(wxId);//用户的openid（要发送给那个用户，通常这里应该动态传进来的）
        wxMssVo.setTemplate_id(templateId);//订阅消息模板id
//        wxMssVo.setPage("pages/index/index");
        wxMssVo.setData(getParam(type,msgList));
        ResponseEntity<String> responseEntity =
                restTemplate.postForEntity(url, wxMssVo, String.class);
        String body = responseEntity.getBody();
        System.err.println(body);
    }

    /**
     * 小程序消息推送
     * @param wxId  用户openId
     * @param templateId  模板id
     * @param page   页面路由
     * @param msgList  消息内容集合
     * @param type   类型
     */
    public static void sendRiderMsg(String wxId, String templateId, List<String> msgList, Integer type){
        String mpToken = getMpRiderToken();
        RestTemplate restTemplate = new RestTemplate();
        String url = "https://api.weixin.qq.com/cgi-bin/message/subscribe/send?access_token=" +mpToken;
        //拼接推送的模版
        WxMssVo wxMssVo = new WxMssVo();
        wxMssVo.setTouser(wxId);//用户的openid（要发送给那个用户，通常这里应该动态传进来的）
        wxMssVo.setTemplate_id(templateId);//订阅消息模板id
//        wxMssVo.setPage("pages/index/index");
        wxMssVo.setData(getParam(type,msgList));
        ResponseEntity<String> responseEntity =
                restTemplate.postForEntity(url, wxMssVo, String.class);
        String body = responseEntity.getBody();
        logger.error("消息推送状态："+body);
    }

    /**
     * 订阅消息模板
     * @param type  类型  1 接单成功通知	  2 订单完成通知
     * @param msgList 对应值
     * @return
     */
    public static Map<String, TemplateParam> getParam(Integer type, List<String> msgList){
        Map<String, TemplateParam> paras = new HashMap<>();
        if(type==1){ //订单状态通知
            //订单号
            paras.put("character_string1", new TemplateParam(msgList.get(0)));
            //订单内容
            paras.put("thing9", new TemplateParam(msgList.get(1)));
            //订单状态
            paras.put("phrase3", new TemplateParam(msgList.get(2)));
            //更新时间
            paras.put("time27", new TemplateParam(msgList.get(3)));
        }else if(type==2){ //订单状态通知
            //订单号
            paras.put("character_string1", new TemplateParam(msgList.get(0)));
            //订单内容
            paras.put("thing9", new TemplateParam(msgList.get(1)));
            //订单状态
            paras.put("phrase3", new TemplateParam(msgList.get(2)));
            //更新时间
            paras.put("time27", new TemplateParam(msgList.get(3)));
        }else if(type==3){ //新订单通知
            //订单号
            paras.put("character_string13", new TemplateParam(msgList.get(0)));
            //订单名称
            paras.put("thing41", new TemplateParam(msgList.get(1)));
            //订单金额
            paras.put("amount15", new TemplateParam(msgList.get(2)));
            //创建时间
            paras.put("time40", new TemplateParam(msgList.get(3)));
        }
        return paras;
    }


    public static void sendMsgGZH(String wxId, String templateId, List<String> msgList, Integer type) {
        String mpToken = getGZHToken();
        RestTemplate restTemplate = new RestTemplate();
        String url = "https://api.weixin.qq.com/cgi-bin/message/template/send?access_token=" + mpToken;
        //拼接推送的模版
        WxMssVo wxMssVo = new WxMssVo();
        wxMssVo.setTouser(wxId);//用户的openid（要发送给那个用户，通常这里应该动态传进来的）
        wxMssVo.setTemplate_id(templateId);//订阅消息模板id
        String value = commonInfoService.findOne(442).getValue();
        if("1".equals(value)){
            Map<String, String> miniprogram = new HashMap<>();
            miniprogram.put("appid",commonInfoService.findOne(45).getValue());
            wxMssVo.setMiniprogram(miniprogram);
        }else{
            wxMssVo.setUrl(commonInfoService.findOne(19).getValue());
        }

        wxMssVo.setData(getGZHParam(type, msgList));
        ResponseEntity<String> responseEntity =
                restTemplate.postForEntity(url, wxMssVo, String.class);
        String body = responseEntity.getBody();
        //System.err.println(body);
        log.info("公众号推送："+body);
    }

    public static void sendRiderMsgGZH(String wxId, String templateId, List<String> msgList, Integer type) {
        String mpToken = getRiderGZHToken();
        RestTemplate restTemplate = new RestTemplate();
        String url = "https://api.weixin.qq.com/cgi-bin/message/template/send?access_token=" + mpToken;
        //拼接推送的模版
        WxMssVo wxMssVo = new WxMssVo();
        wxMssVo.setTouser(wxId);//用户的openid（要发送给那个用户，通常这里应该动态传进来的）
        wxMssVo.setTemplate_id(templateId);//订阅消息模板id
        String value = commonInfoService.findOne(442).getValue();
        if("1".equals(value)){
            Map<String, String> miniprogram = new HashMap<>();
            miniprogram.put("appid",commonInfoService.findOne(248).getValue());
            wxMssVo.setMiniprogram(miniprogram);
        }else{
            wxMssVo.setUrl(commonInfoService.findOne(22).getValue());
        }
        wxMssVo.setData(getGZHParam(type, msgList));
        ResponseEntity<String> responseEntity =
                restTemplate.postForEntity(url, wxMssVo, String.class);
        String body = responseEntity.getBody();
        //System.err.println(body);
        log.info("公众号推送："+body);
    }

    public static Map<String, TemplateParam> getGZHParam(Integer type, List<String> msgList) {
        Map<String, TemplateParam> paras = new HashMap<>();
        if (type == 1) {
            paras.put("character_string10", new TemplateParam(msgList.get(0)));
            paras.put("thing7", new TemplateParam(msgList.get(1)));
            paras.put("amount9", new TemplateParam(msgList.get(2)));
            paras.put("time13", new TemplateParam(msgList.get(3)));
        }else if(type==2){
            paras.put("character_string9", new TemplateParam(msgList.get(0)));
            paras.put("thing31", new TemplateParam(msgList.get(1)));
            paras.put("amount8", new TemplateParam(msgList.get(2)));
            paras.put("time4", new TemplateParam(msgList.get(3)));
        }else if(type==3){
            paras.put("character_string1", new TemplateParam(msgList.get(0)));
            paras.put("thing3", new TemplateParam(msgList.get(1)));
            paras.put("time2", new TemplateParam(msgList.get(2)));
        }else if(type==4){
            paras.put("character_string6", new TemplateParam(msgList.get(0)));
            paras.put("thing12", new TemplateParam(msgList.get(1)));
            paras.put("time4", new TemplateParam(msgList.get(2)));
        }else if(type==5){
            paras.put("character_string8", new TemplateParam(msgList.get(0)));
            paras.put("thing19", new TemplateParam(msgList.get(1)));
            paras.put("time18", new TemplateParam(msgList.get(2)));
        }else if(type==6){
            paras.put("character_string1", new TemplateParam(msgList.get(0)));
            paras.put("thing6", new TemplateParam(msgList.get(1)));
            paras.put("time4", new TemplateParam(msgList.get(2)));
        }
        return paras;
    }





}